@@ -1,5 +1,5 @@
 /*******************************************************************************
-   This file is part of libTMCG.
+   This file is part of LibTMCG.
 
      Christian Schindelhauer: 'A Toolbox for Mental Card Games',
      Technical Report A-98-14, University of L{\"u}beck, 1998.
@@ -14,20 +14,20 @@
 
      Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions', 2002
 
- Copyright (C) 2004, 2005, 2006  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
 
-   libTMCG is free software; you can redistribute it and/or modify
+   LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
-   libTMCG is distributed in the hope that it will be useful,
+   LibTMCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with libTMCG; if not, write to the Free Software
+   along with LibTMCG; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
@@ -111,6 +111,7 @@ bool TMCG_PublicKey::check
 		// check, whether m \not\in FP (fermat primes: m = 2^k + 1)
 		mpz_set(foo, m);
 		mpz_sub_ui(foo, foo, 1L);
+// FIXME: is m correct here? (not foo) 
 		unsigned long int k = mpz_sizeinbase(m, 2L);
 		mpz_set_ui(bar, 2L);
 		mpz_pow_ui(bar, bar, k);
@@ -137,12 +138,16 @@ bool TMCG_PublicKey::check
 					throw false;
 			}
 		}
+
+		// abort, if non-NIZK key
+		if (type.find("NIZK", 0) == type.npos)
+			throw true;
 		
 		// check magic of NIZK
 		if (!TMCG_ParseHelper::cm(s, "nzk", '^'))
 			throw false;
 		
-		// initalize NIZK proof input
+		// initialize NIZK proof input
 		std::ostringstream input;
 		input << m << "^" << y;
 		
@@ -150,7 +155,8 @@ bool TMCG_PublicKey::check
 		if (TMCG_ParseHelper::gs(s, '^').length() == 0)
 			throw false;
 		
-		stage1_size = strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
+		stage1_size = 
+		    std::strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
 		if ((*ec != '\0') || (stage1_size <= 0) || (!TMCG_ParseHelper::nx(s, '^')))
 			throw false;
 		
@@ -188,7 +194,8 @@ bool TMCG_PublicKey::check
 		// get security parameter of STAGE2
 		if (TMCG_ParseHelper::gs(s, '^').length() == 0)
 			throw false;
-		stage2_size = strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
+		stage2_size = 
+		    std::strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
 		if ((*ec != '\0') || (stage2_size <= 0) || (!TMCG_ParseHelper::nx(s, '^')))
 			throw false;
 		
@@ -239,7 +246,8 @@ bool TMCG_PublicKey::check
 		// get security parameter of STAGE3
 		if (TMCG_ParseHelper::gs(s, '^').length() == 0)
 			throw false;
-		stage3_size = strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
+		stage3_size = 
+		    std::strtoul(TMCG_ParseHelper::gs(s, '^').c_str(), &ec, 10);
 		if ((*ec != '\0') || (stage3_size <= 0) || (!TMCG_ParseHelper::nx(s, '^')))
 			throw false;
 		
@@ -358,7 +366,8 @@ size_t TMCG_PublicKey::keyid_size
 	
 	// extract the size
 	char *ec;
-	size_t size = strtoul(s.substr(2, s.find("^") - 2).c_str(), &ec, 10);
+	size_t size = 
+	    std::strtoul(s.substr(2, s.find("^") - 2).c_str(), &ec, 10);
 	if ((*ec != '\0') || (size != (s.length() - s.find("^") - 1)))
 		return 0;
 	
