@@ -1,5 +1,5 @@
 /*******************************************************************************
-   This file is part of libTMCG.
+   This file is part of LibTMCG.
 
      Christian Schindelhauer: 'A Toolbox for Mental Card Games',
      Technical Report A-98-14, University of L{\"u}beck, 1998.
@@ -14,20 +14,20 @@
 
      Dan Boneh: 'Simplified OAEP for the RSA and Rabin Functions', 2002
 
- Copyright (C) 2004, 2005, 2006  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2004, 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
 
-   libTMCG is free software; you can redistribute it and/or modify
+   LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
-   libTMCG is distributed in the hope that it will be useful,
+   LibTMCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with libTMCG; if not, write to the Free Software
+   along with LibTMCG; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
@@ -44,14 +44,14 @@ TMCG_SecretKey::TMCG_SecretKey
 
 TMCG_SecretKey::TMCG_SecretKey
 	(const std::string& n, const std::string& e,
-	unsigned long int keysize):
+	unsigned long int keysize, bool nizk_key):
 		name(n), email(e)
 {
 	mpz_init(m), mpz_init(y), mpz_init(p), mpz_init(q);
 	mpz_init(y1), mpz_init(m1pq), mpz_init(gcdext_up), mpz_init(gcdext_vq),
 		mpz_init(pa1d4), mpz_init(qa1d4);
 	
-	generate(keysize);
+	generate(keysize, nizk_key);
 }
 
 TMCG_SecretKey::TMCG_SecretKey
@@ -92,7 +92,7 @@ TMCG_SecretKey& TMCG_SecretKey::operator =
 }
 
 void TMCG_SecretKey::generate
-	(unsigned long int keysize)
+	(unsigned long int keysize, bool nizk_key)
 {
 	mpz_t foo, bar;
 	
@@ -101,7 +101,10 @@ void TMCG_SecretKey::generate
 	
 	// set type of key
 	std::ostringstream t;
-	t << "TMCG/RABIN_" << keysize << "_NIZK";
+	if (nizk_key)
+		t << "TMCG/RABIN_" << keysize << "_NIZK";
+	else
+		t << "TMCG/RABIN_" << keysize;
 	type = t.str();
 	
 	// generate appropriate primes for RABIN encryption with SAEP
@@ -143,7 +146,7 @@ void TMCG_SecretKey::generate
 	
 	// pre-compute non-persistent values
 	precompute();
-	
+
 	// Rosario Gennaro, Daniele Micciancio, Tal Rabin:
 	// 'An Efficient Non-Interactive Statistical Zero-Knowledge
 	// Proof System for Quasi-Safe Prime Products',
@@ -159,7 +162,7 @@ void TMCG_SecretKey::generate
 	// STAGE1: m Square Free
 	// soundness error probability \le d^{-TMCG_KEY_NIZK_STAGE1}
 	nizk2 << TMCG_KEY_NIZK_STAGE1 << "^";
-	for (size_t stage1 = 0; stage1 < TMCG_KEY_NIZK_STAGE1; stage1++)
+	for (size_t stage1 = 0; (stage1 < TMCG_KEY_NIZK_STAGE1) && nizk_key; stage1++)
 	{
 		// common random number foo \in Z^*_m (build from hash function g)
 		do
@@ -182,7 +185,7 @@ void TMCG_SecretKey::generate
 	// STAGE2: m Prime Power Product
 	// soundness error probability \le 2^{-TMCG_KEY_NIZK_STAGE2}
 	nizk2 << TMCG_KEY_NIZK_STAGE2 << "^";
-	for (size_t stage2 = 0; stage2 < TMCG_KEY_NIZK_STAGE2; stage2++)
+	for (size_t stage2 = 0; (stage2 < TMCG_KEY_NIZK_STAGE2) && nizk_key; stage2++)
 	{
 		// common random number foo \in Z^*_m (build from hash function g)
 		do
@@ -226,7 +229,7 @@ void TMCG_SecretKey::generate
 	// STAGE3: y \in NQR^\circ_m
 	// soundness error probability \le 2^{-TMCG_KEY_NIZK_STAGE3}
 	nizk2 << TMCG_KEY_NIZK_STAGE3 << "^";
-	for (size_t stage3 = 0; stage3 < TMCG_KEY_NIZK_STAGE3; stage3++)
+	for (size_t stage3 = 0; (stage3 < TMCG_KEY_NIZK_STAGE3) && nizk_key; stage3++)
 	{
 		// common random number foo \in Z^\circ_m (build from hash function g)
 		do
@@ -519,10 +522,8 @@ std::string TMCG_SecretKey::sign
 		p, q, m, gcdext_up, gcdext_vq, pa1d4, qa1d4);
 	
 	// choose a square root randomly (one out-of four)
-	mpz_srandomb(foo, 2L);
-	
 	std::ostringstream ost;
-	ost << "sig|" << keyid() << "|" << foo_sqrt[mpz_get_ui(foo) % 4] << "|";
+	ost << "sig|" << keyid() << "|" << foo_sqrt[mpz_srandom_mod(4)] << "|";
 	mpz_clear(foo), mpz_clear(foo_sqrt[0]), mpz_clear(foo_sqrt[1]),
 		mpz_clear(foo_sqrt[2]), mpz_clear(foo_sqrt[3]);
 	
